/** Put here CSS that should apply to both public and editor screens. **/

body {
    padding-top: 30px;
}

body.logged-in {
    padding-top: 60px;
}

@media (max-width: 980px) {
    body.logged-in {
        padding-top: 0;
    }
}
