/** Put here CSS that should apply to public screens only. **/

a.site-title {
    padding-bottom: 18px;
    font-size: 54px;
    display: block;
    color: #333;
    text-decoration: none;
    line-height: 1;
    letter-spacing: -1px;
}

#search {
    position: absolute;
    right: 50px;
    top: 50px;
}

#search input {
    margin: 0;
}

@media (max-width: 980px) {
    a.site-title {
        font-size: 30px;
    }
}
